// Import std and git2 crate for downloading, building and launching client part 
use std::process::{Command, Stdio};
use std::fs::File;
use std::{fs, path::Path, path::PathBuf};
use git2::Repository;

// download client patch from the distant git repository
pub fn zzz_download_client_patch() {

    let zzz_client_dir_path = "zzz_client_patch";
    if let Err(e) = fs::create_dir_all(zzz_client_dir_path) {
        eprintln!("Error creating folder: {}", e);
        return;
    }

    let zzz_game_dir_path = "zzz_game";
    if let Err(e) = fs::create_dir_all(zzz_game_dir_path) {
        eprintln!("Error creating folder: {}", e);
        return;
    }
    
    let repo_url = "https://git.xeondev.com/traffic95/trigger-patch";

    let path = Path::new(zzz_client_dir_path);

    match Repository::clone(repo_url, path) {
        Ok(_) => {
            println!("✅ Client Repo cloned at {}", zzz_client_dir_path);
        }
        Err(e) => {
            eprintln!("Error during Client Patch downloading : {}", e);
        }
    }
}

// Build the Client from source
pub fn zzz_build_client_patch() {

    let client_patch_log_file_path = "client_patch_log.txt";
    let client_patch_log_file = File::create(client_patch_log_file_path).expect("Unable to create log file");
    let client_patch_lof_file_clone = client_patch_log_file.try_clone().expect("Unable to clone log file handle");
    let mut child = Command::new("cargo")
        .arg("+nightly")
        .arg("build")
        .arg("--target")
        .arg("x86_64-pc-windows-msvc")
        .current_dir("zzz_client_patch")
        .stdout(Stdio::from(client_patch_log_file))
        .stderr(Stdio::from(client_patch_lof_file_clone))
        .spawn()
        .expect("❌ Error during the launch of cargo build");

    let status = child.wait().expect("Error during process waiting");

    if status.success() {
        println!("✅ Client built!");
    } else {
        println!("❌ Build failed with status: {}", status);
    }
}

// Move the exe and dll of the Client patch to the game folder, and launch it
pub fn zzz_client_patch_loaded() {
    let source_dir = PathBuf::from("zzz_client_patch/target/x86_64-pc-windows-msvc/debug");
    let launcher = source_dir.join("launcher.exe");
    let dll = source_dir.join("trigger.dll");

    let dest_dir = PathBuf::from("zzz_game");

    fs::copy(&launcher, dest_dir.join("launcher.exe")).expect("Failed to copy launcher.exe");
    fs::copy(&dll, dest_dir.join("trigger.dll")).expect("Failed to copy trigger.dll");

    let launcher_exec_path = PathBuf::from("zzz_game/launcher.exe");

    if !launcher_exec_path.exists() {
        eprintln!("❌ launcher.exe not found, Make sure the build was successful and the files were copied.");
        return;
    }

    match Command::new(&launcher_exec_path)
        .stdout(Stdio::inherit())
        .stderr(Stdio::inherit())
        .spawn()
    {
        Ok(mut child) => {
            println!("launcher.exe is starting");
            let _ = child.wait();
        }
        Err(e) => {
            eprintln!("❌ Can't Launch the launcher : {}",e )
        }
    }
}