// Import std and git2 crate for downloading, building and launching servers part 
use std::{
    collections::HashMap,
    fs::{self, File},
    io::{BufRead, BufReader},
    path::Path,
    process::{Command, Stdio},
    sync::{Arc, Mutex},
    thread,
};

use git2::Repository;

// Funtion for download the server from the distant Git repository
pub fn zzz_download_server() {
    let zzz_dir_path = "zzz_ps";
    if let Err(e) = fs::create_dir_all(zzz_dir_path) {
        eprintln!("Error creating folder: {}", e);
        return;
    }

    let repo_url = "https://git.xeondev.com/traffic95/trigger-rs";
    let path = Path::new(zzz_dir_path);

    match Repository::clone(repo_url, path) {
        Ok(_) => println!("✅ Server Repo cloned at {}", zzz_dir_path),
        Err(e) => eprintln!("❌ Clone failed: {}", e),
    }
}

// Function for building the server via cargo command build
pub fn zzz_build_server() {
    let log_path = "server_log.txt";
    let log_file = File::create(log_path).expect("Cannot create log file");
    let log_file_clone = log_file.try_clone().expect("Cannot clone log handle");

    let mut child = Command::new("cargo")
        .arg("build")
        .current_dir("zzz_ps")
        .stdout(Stdio::from(log_file))
        .stderr(Stdio::from(log_file_clone))
        .spawn()
        .expect("Error launching build");

    let status = child.wait().expect("Error waiting for process");

    if status.success() {
        println!("✅ Server built!");
    } else {
        println!("❌ Build failed with status: {}", status);
    }
}

// List every servers part in the Launcher UI
pub fn list_available_servers() -> Vec<String> {
    let server_dir = Path::new("zzz_ps/target/debug");
    let expected = [
        "trigger-muip-server.exe",
        "trigger-hall-server.exe",
        "trigger-gate-server.exe",
        "trigger-game-server.exe",
        "trigger-dispatch-server.exe",
        "trigger-battle-server.exe",
    ];

    expected
        .iter()
        .filter_map(|name| {
            let path = server_dir.join(name);
            if path.exists() {
                Some(name.to_string())
            } else {
                None
            }
        })
        .collect()
}

// Launch the servers inside of every designed tabs with logs
pub fn launch_server_with_logs(
    exe_name: &str,
    logs: Arc<Mutex<HashMap<String, Vec<String>>>>,
) {
    let path = format!("zzz_ps/target/debug/{}", exe_name);
    let mut child = match Command::new(&path)
        .stdout(Stdio::piped())
        .stderr(Stdio::piped())
        .spawn()
    {
        Ok(c) => c,
        Err(e) => {
            eprintln!("❌ Failed to launch {}: {}", exe_name, e);
            return;
        }
    };

    let stdout = child.stdout.take().unwrap();
    let stderr = child.stderr.take().unwrap();

    let logs_clone = Arc::clone(&logs);
    let name = exe_name.to_string();
    thread::spawn(move || {
        let reader = BufReader::new(stdout);
        for line in reader.lines().flatten() {
            let mut logs = logs_clone.lock().unwrap();
            logs.entry(name.clone()).or_default().push(format!("📤 {}", line));
        }
    });

    let logs_clone = Arc::clone(&logs);
    let name = exe_name.to_string();
    thread::spawn(move || {
        let reader = BufReader::new(stderr);
        for line in reader.lines().flatten() {
            let mut logs = logs_clone.lock().unwrap();
            logs.entry(name.clone()).or_default().push(format!("📕 {}", line));
        }
    });
}
